# Search for gravitational-lensing signatures in the full third observing run of the LIGO–Virgo network
## Python script for "Lensing rates" | Table 2

##########
#"Search for gravitational-lensing signatures in the full third observing run of the LIGO–Virgo network" is the second paper from the LIGO, 
# Virgo, and KAGRA (LVK) Collaborations searching for possible lensing features from gravitational waves during the full third observing run
# (O3a + O3b) which lasted from 1st April # 2019 to 27th March 2020.
##########

##############
# What is lensing of gravitational waves?
##############

# General relativity tells us that gravity curves space and time, affecting everything that carries mass or energy, and gravitational waves 
# are no exception. Therefore, we expect that as they travel cosmological distances, gravitational waves will inevitably encounter massive 
# astrophysical objects that would act as lenses, bending their trajectories and even distorting the shapes of their signals. In the paper, 
# we have implemented multiple analyses to search for the still undetected lensing signatures in the gravitational-wave signals detected in 
# O3a+b. 

# This code is part of the data release for the paper. It generates the Table 2 showing the lensing rates for different types of lenses and for two cases based on whether the first brightest image or the second brightest image meets the detection threshold.

import numpy as np


rstoch_l,rstoch_h=np.loadtxt("Rzstoch_modelB.txt",unpack=1)

print("\n\n")
print("Relative lensing rates (x 1e-4)")
print("Model  Gal_Doubles  Gal_Singles   Clus_Doubles  Clus_Singles")
print("=============================================================")
print("GWTC-3+Stochastic  &  %.1f-%.1f  & %.1f-%.1f  & %.1f-%.1f & %.1f-%.1f \n"%(rstoch_l[0],rstoch_h[0],rstoch_l[1],rstoch_h[1],rstoch_l[2],rstoch_h[2],rstoch_l[3],rstoch_h[3]))
print("\n\n")

fp=open("O3b_lensing_paper_Table2.txt","w")

fp.write("Merger Rate Density  & \multicolumn{2}{c} {Galaxies}  & \multicolumn{2}{c}  {Galaxy Clusters}\\\n")
fp.write("Model &  $R_{\\rm D}$ & $R_{\\rm S}$ & $R_{\\rm D}$ & $R_{\\rm S}$ \\\n")
fp.write("\hline\hline\n")
fp.write("GWTC-3+Stochastic  &  %.1f$-$%.1f $\\times 10^{-4}$ & %.1f$-$%.1f $\\times 10^{-4}$ & %.1f$-$%.1f $\\times 10^{-4}$ & %.1f$-$%.1f $\\times 10^{-4}$ \n"%(rstoch_l[0],rstoch_h[0],rstoch_l[1],rstoch_h[1],rstoch_l[2],rstoch_h[2],rstoch_l[3],rstoch_h[3])) 
fp.write("\hline")
